#load packages
library(class)
library(MASS)
library(kohonen)
library(pls)
library(ChemometricsWithRData)
library(ChemometricsWithR)
library(prospectr)
library(zoom)
library(plyr)
getwd()
setwd("D:\\aguatamegarcia\\My Documents\\STOICISM\\DIATOMACEOUS EARTH\\Diatomite_Diffuse")


### 1- DATA IMPORT

#load dataset
## load spectral data
temp <- list.files(pattern = "*.asp")
ref <- read.table("specref.asp")		#specref no meaningful data, only for formating purpose
points <- ref[1, ]
wavi <- ref[2, ]
wavf <- ref[3, ]
res <- ref[4, ]
step <- ((wavi - wavf) / (points - 1) * (-1))
wavenum <- t(seq(wavi, wavf, step))
wavelen.temp <- (10000/wavenum)
wavelen <-(format(round(wavelen.temp, digits = 3)))

#Compilation of spectra in a single table
rawspectra <- do.call(rbind, sapply(temp, read.table, sep = "", skip = 6)) 
rawspectra <- rawspectra[1:90, ]
colnames(rawspectra) <-c(wavelen)

# Sample average every 5 samples
diatomite <- (rowsum(rawspectra, group = rep(1:18, each = 5), reorder = FALSE))/5 

matplot(as.numeric(colnames(D)),t(D), type = "l", lty = 1, col = 1:18)
legend("topright", lty = 1, legend = c(rownames(diatomite)), col = 1:18)

#Import XRF data
xrf <- read.csv2("XRF.csv")
xrf <- xrf[ ,2:37]

cutoff <- read.csv2("cutoff.csv")
CaO <- read.table("cao.txt")

#name all samples
samplenames <- read.table("names.txt")
rownames(xrf) <- c(t(samplenames))
rownames(diatomite) <- c(t(samplenames))
rownames(cutoff) <-c(t(samplenames))
rownames(CaO) <- c(t(samplenames))



### 2- DATA PRE-PROCESSING
#2.1- SG filtering
diatomite.sg <-savitzkyGolay(diatomite, p = 3, w = 55, m = 0)
matplot(as.numeric(colnames(diatomite.sg)),t(diatomite.sg), type = "l", lty = 1)


#2.2 Standard Normal Variate (SNV)
diatomite.snv <-standardNormalVariate(X = diatomite.sg)
matplot(as.numeric(colnames(diatomite.snv)),t(diatomite.snv), type = "l", lty = 1)
#spectra.snv.c <- (spectra.snv + (abs(min(spectra.snv)))) #(fix offset)

#2.3- Derivatives
spectra.d1 <- t(diff(t(spectra.snv), differences = 1))
spectra.d2 <- t(diff(t(spectra.snv), differences = 2))
plot((as.numeric(colnames(spectra.d1))),spectra.d1[1, ], type = "l")
plot((as.numeric(colnames(spectra.d2))),spectra.d2[1, ], type = "l")
lines(as.numeric(colnames(spectra.d2)), spectra.d2[1, ], col = "red")


### 3- SUBSET SPECTRA

##LONG WAVE [6-8um]
temp.wavi.l <-grep("6.000", colnames(diatomite.sg))
temp.wavf.l <-grep("8.000",colnames(diatomite.sg))
diatomite.long <- diatomite.sg[ ,(temp.wavi.l[1]:temp.wavf.l[1])]



#Choose SNV,  CR or scale

diatomite.long.cr <-continuumRemoval(X = diatomite.long, type = "R")
#diatomite.long.d1 <- t(diff(t(diatomite.long.cr), differences = 1))

matplot(as.numeric(colnames(diatomite.long)),t(diatomite.long), type = "l", lty = 1)
diatomite.sc <- scale(diatomite.long.cr) #scales data
diatomite.sc.plot <- (matplot(as.numeric(colnames(diatomite.sc)), t(diatomite.sc), 
                            type = "l", lty = 1, xlab = "Wavelength (nm)", ylab = "Reflectance"))

### 4- PCA

diatomite.svd <-svd(diatomite.sc[ ,2:894]) # Singular Value Decomposition
diatomite.scores <-diatomite.svd$u%*%diag(diatomite.svd$d) # Calculates the scores based on the U values and the importance(d)
diatomite.loadings <- diatomite.svd$v #Creates the loadings
diatomite.vars <-diatomite.svd$d^2 / (nrow(diatomite.long)-1)
diatomite.totalvar <- sum(diatomite.vars)
diatomite.relvars <- diatomite.vars / diatomite.totalvar
variances <- 100*round(diatomite.relvars, digits = 3)
variances[1:5]			#All var lines: step-wise variance calculation

par(mfrow = c(2,1))

#Scores plot
scoresplot <- plot(diatomite.scores[,1:2], type = "n", 
                   xlab = paste("PC1 (", variances[1],"%)", sep = ""), 
                   ylab = paste("PC2 (", variances[2],"%)", sep = ""))
abline(h = 0, v = 0, col = "gray")
points(diatomite.scores[,1:2], col = cutoff$Class)
#text(diatomite.scores[ 2:3], labels = rownames(diatomite))


#loadings plot
loadings.plot <- (matplot(as.numeric(colnames(diatomite.long[ ,2:894])), diatomite.loadings[ ,1], 
                          type = "l", lty = 1, xlab = "Wavelength (nm)", ylab = "Loadings PC1"))

### 5- PLS

diatomite.data <- data.frame(calcium = CaO, spectra = I(diatomite.long.cr[ ,2:894]))
diatomite.data <- data.frame(SiO = SiO, spectra = I(diatomite.sc[ ,2:894]))

diatomite.data.m <-plsr(diatomite.data$V1~diatomite.data$spectra, ncomp = 8, data = diatomite.data, validation = "LOO")	#PLS model calculated for 8 components, validation leave-one-out(LOO)
summary(diatomite.data.m)
plot(RMSEP(diatomite.data.m), legendpos = "topright")
plot(diatomite.data.m, ncomp = 5, asp = 1, line = TRUE)
